@@ -34,7 +34,8 @@ void	plugin_x11_setup_windowed	(SwfmozPlayer *	      player,
 					 int		      width,
 					 int		      height);
 void	plugin_x11_teardown		(SwfmozPlayer *	      player);
-
+void	plugin_x11_handle_event		(SwfmozPlayer *	      player,
+					 XEvent *	      event);
 
 G_END_DECLS
 #endif
