@@ -168,6 +168,7 @@ swfmoz_dialog_get_media_page (SwfmozDialog *dialog)
   GtkWidget *vbox, *align, *widget;
   GtkCellRenderer *renderer;
   GtkTreeViewColumn *column;
+  GtkTreePath *path;
   
   vbox = gtk_vbox_new (FALSE, 3);
   
@@ -190,19 +191,18 @@ swfmoz_dialog_get_media_page (SwfmozDialog *dialog)
   gtk_tree_view_column_set_sort_column_id (column, SWFMOZ_LOADER_COLUMN_TYPE);
   gtk_tree_view_append_column (GTK_TREE_VIEW (widget), column);
 
-  renderer = gtk_cell_renderer_progress_new ();
-  column = gtk_tree_view_column_new_with_attributes ("Done", renderer,
-    "value", SWFMOZ_LOADER_COLUMN_PERCENT_LOADED, NULL);
+  renderer = gtk_cell_renderer_text_new ();
+  column = gtk_tree_view_column_new_with_attributes ("Status", renderer,
+    "text", SWFMOZ_LOADER_COLUMN_STATUS, NULL);
   gtk_tree_view_column_set_resizable (column, TRUE);
-  gtk_tree_view_column_set_sort_column_id (column, SWFMOZ_LOADER_COLUMN_PERCENT_LOADED);
+  gtk_tree_view_column_set_sort_column_id (column, SWFMOZ_LOADER_COLUMN_STATUS);
   gtk_tree_view_append_column (GTK_TREE_VIEW (widget), column);
 
-  renderer = gtk_cell_renderer_toggle_new ();
-  column = gtk_tree_view_column_new_with_attributes ("Error", renderer,
-    "active", SWFMOZ_LOADER_COLUMN_ERROR, NULL);
-  gtk_tree_view_column_set_resizable (column, TRUE);
-  gtk_tree_view_column_set_sort_column_id (column, SWFMOZ_LOADER_COLUMN_ERROR);
-  gtk_tree_view_append_column (GTK_TREE_VIEW (widget), column);
+  gtk_tree_selection_set_mode (gtk_tree_view_get_selection
+                               (GTK_TREE_VIEW (widget)), GTK_SELECTION_BROWSE);
+  path = gtk_tree_path_new_from_indices (0, -1);
+  gtk_tree_view_set_cursor (GTK_TREE_VIEW (widget), path, NULL, FALSE);
+  gtk_tree_path_free (path);
 
   align = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (align),
@@ -210,12 +210,12 @@ swfmoz_dialog_get_media_page (SwfmozDialog *dialog)
   gtk_container_add (GTK_CONTAINER (align), widget);
   gtk_box_pack_start (GTK_BOX (vbox), align, TRUE, TRUE, 0);
 
-  widget = gtk_button_new_with_mnemonic ("_Save media as");
+  widget = gtk_button_new_with_mnemonic ("_Save...");
   g_signal_connect (widget, "clicked", G_CALLBACK (swfmoz_dialog_save_media), dialog);
   align = gtk_alignment_new (1.0, 0.5, 0.0, 0.0);
   gtk_container_add (GTK_CONTAINER (align), widget);
   gtk_box_pack_end (GTK_BOX (vbox), align, FALSE, TRUE, 0);
-  
+
   return vbox;
 }
 
