@@ -31,10 +31,14 @@
 /*** menu ***/
 
 static void
-swfmoz_player_menu_playing_toggled (GtkCheckMenuItem *item, SwfdecGtkPlayer* player)
+swfmoz_player_menu_playing_toggled (GtkCheckMenuItem *item, SwfmozPlayer* player)
 {
-  if (swfdec_gtk_player_get_playing (player) != gtk_check_menu_item_get_active (item))
-    swfdec_gtk_player_set_playing (player, gtk_check_menu_item_get_active (item));
+  gboolean menu_item_active = gtk_check_menu_item_get_active (item);
+
+  if (swfdec_gtk_player_get_playing (SWFDEC_GTK_PLAYER (player)) != menu_item_active) {
+      swfdec_gtk_player_set_playing (SWFDEC_GTK_PLAYER (player), menu_item_active);
+      swfmoz_config_set_autoplay (player->config, swfdec_player_get_url (SWFDEC_PLAYER (player)), menu_item_active);
+  }
 }
 
 static void
@@ -67,6 +71,7 @@ swfmoz_player_menu_about (GtkMenuItem *item, SwfmozPlayer *player)
     "Benjamin Otte <otte@gnome.org>",
     "David Schleef <ds@schleef.org>",
     "Eric Anholt <eric@anholt.net>",
+    "Pekka Lampila <pekka.lampila@iki.fi>",
     NULL,
   };
   static const char *artists[] = {
@@ -149,16 +154,31 @@ swfmoz_player_idle_redraw (gpointer playerp)
   GdkRegion *region;
 
   region = player->repaint;
-  if (region) {
-    player->repaint = NULL;
-    g_source_destroy (player->repaint_source);
-    g_source_unref (player->repaint_source);
-    player->repaint_source = NULL;
-    swfmoz_player_render (player, region);
-    gdk_region_destroy (region);
-  }
+  player->repaint = NULL;
+  g_source_unref (player->repaint_source);
+  player->repaint_source = NULL;
+  if (player->windowless) {
+    NPRect rect;
+    GdkRectangle *rectangles;
+    int i, n_rectangles;
 
-  return TRUE;
+    g_assert (player->repaint == NULL);
+    
+    gdk_region_get_rectangles (region, &rectangles, &n_rectangles);
+
+    for (i = 0; i < n_rectangles; i++) {
+      rect.left = rectangles[i].x;
+      rect.top = rectangles[i].y;
+      rect.right = rectangles[i].x + rectangles[i].width;
+      rect.bottom = rectangles[i].y + rectangles[i].height;
+      plugin_invalidate_rect (player->instance, &rect);
+    }
+  } else {
+    swfmoz_player_render (player, NULL, region);
+  }
+  gdk_region_destroy (region);
+
+  return FALSE;
 }
 
 static void
@@ -177,49 +197,78 @@ swfmoz_player_redraw (SwfmozPlayer *player, const SwfdecRectangle *extents,
     gdk_region_union_with_rect (region, (GdkRectangle *) &rects[i]);
   }
 
-  if (player->windowless) {
-    NPRect rect;
-    GdkRectangle *rectangles;
-    int n_rectangles;
-
-    g_assert (player->repaint == NULL);
-    
-    gdk_region_get_rectangles (region, &rectangles, &n_rectangles);
-
-    for (i = 0; i < (guint) n_rectangles; i++) {
-      rect.left = rectangles[i].x;
-      rect.top = rectangles[i].y;
-      rect.right = rectangles[i].x + rectangles[i].width;
-      rect.bottom = rectangles[i].y + rectangles[i].height;
-      plugin_invalidate_rect (player->instance, &rect);
-    }
-    gdk_region_destroy (region);
+  if (player->repaint_source) {
+    g_assert (player->repaint);
   } else {
-    if (player->repaint_source) {
-      g_assert (player->repaint);
-    } else {
-      GSource *source = g_idle_source_new ();
-      player->repaint_source = source;
-      g_source_set_priority (source, GDK_PRIORITY_REDRAW);
-      g_source_set_callback (source, swfmoz_player_idle_redraw, player, NULL);
-      g_source_attach (source, player->context);
-      player->repaint = region;
-    }
+    GSource *source = g_idle_source_new ();
+    player->repaint_source = source;
+    g_source_set_priority (source, SWFDEC_GTK_PRIORITY_REDRAW);
+    g_source_set_callback (source, swfmoz_player_idle_redraw, player, NULL);
+    g_source_attach (source, player->context);
+    player->repaint = region;
   }
 }
 
-static void
-swfmoz_player_launch (SwfmozPlayer *player, SwfdecLoaderRequest request, 
-    const char *url, const char *target, SwfdecBuffer *data, gpointer unused)
+static SwfdecBuffer *
+swfmoz_player_format_headers (guint header_count, const char **header_names,
+    const char **header_values)
 {
-  if (request == SWFDEC_LOADER_REQUEST_POST) {
-    if (data) {
-      plugin_post_url (player->instance, url, target, 
-	  (const char *) data->data, data->length);
-    } else {
-      plugin_post_url (player->instance, url, target, NULL, 0);
-    }
+  GString *string;
+  guint i;
+  gsize len;
+
+  g_return_val_if_fail (header_count == 0 || header_names != NULL, NULL);
+  g_return_val_if_fail (header_count == 0 || header_values != NULL, NULL);
+
+  string = g_string_new ("");
+  for (i = 0; i < header_count; i++) {
+    g_string_append (string, header_names[i]);
+    g_string_append (string, ": ");
+    g_string_append (string, header_values[i]);
+    g_string_append (string, "\n");
+  }
+  g_string_append (string, "\n\n");
+
+  len = string->len;
+  return swfdec_buffer_new_for_data (
+      (unsigned char *)g_string_free (string, FALSE), len);
+}
+
+/* Returns a new buffer with the headers prepended, suitable for passing to
+ * plugin_post_url_notify */
+SwfdecBuffer *
+swfmoz_player_add_headers (SwfdecBuffer *data, guint header_count,
+    const char **header_names, const char **header_values)
+{
+  SwfdecBufferQueue *queue;
+  SwfdecBuffer *buffer;
+
+  g_return_val_if_fail (data != NULL, NULL);
+
+  queue = swfdec_buffer_queue_new ();
+  swfdec_buffer_queue_push (queue, swfmoz_player_format_headers (
+	header_count, header_names, header_values));
+  swfdec_buffer_queue_push (queue, swfdec_buffer_ref (data));
+
+  buffer = swfdec_buffer_queue_pull (queue,
+      swfdec_buffer_queue_get_depth (queue));
+  swfdec_buffer_queue_unref (queue);
+
+  return buffer;
+}
+
+static void
+swfmoz_player_launch (SwfmozPlayer *player, const char *url,
+    const char *target, SwfdecBuffer *data, guint header_count,
+    const char **header_names, const char **header_values,  gpointer unused)
+{
+  if (data) {
+  // use the notify version of the function so headers can be sent
+  // we don't actually care about the notify
+    plugin_post_url_notify (player->instance, url, target,
+	(const char *)data->data, data->length, NULL);
   } else {
+    // FIXME: no way to send headers here?
     plugin_get_url (player->instance, url, target);
   }
 }
@@ -240,7 +289,8 @@ swfmoz_player_update_cursor (SwfmozPlayer *player)
   SwfdecMouseCursor swfcursor;
   GdkCursor *cursor;
 
-  if (window == NULL)
+  /* FIXME: make this work for windowless mode */
+  if (window == NULL || player->windowless)
     return;
   display = gdk_drawable_get_display (window);
 
@@ -288,6 +338,35 @@ swfmoz_player_update_cursor (SwfmozPlayer *player)
   }
 }
 
+static void
+swfmoz_player_update_background (SwfmozPlayer *player)
+{
+  GdkWindow *window = player->target;
+  GdkColor bgcolor;
+  guint bg;
+
+  if (player->windowless || window == NULL)
+    return;
+
+  bg = swfdec_player_get_background_color (SWFDEC_PLAYER (player));
+  bgcolor.red = 0x101 * ((bg >> 16) & 0xFF);
+  bgcolor.green = 0x101 * ((bg >> 8) & 0xFF);
+  bgcolor.blue = 0x101 * (bg & 0xFF);
+  gdk_rgb_find_color (gdk_drawable_get_colormap (window), &bgcolor);
+  gdk_window_set_background (window, &bgcolor);
+}
+
+static void
+swfmoz_player_fullscreen_destroyed (GtkWidget *widget, SwfmozPlayer *player)
+{
+  player->fullscreen = NULL;
+  if (swfdec_player_get_fullscreen (SWFDEC_PLAYER (player))) {
+    swfdec_player_set_focus (SWFDEC_PLAYER (player), TRUE);
+    swfdec_player_key_press (SWFDEC_PLAYER (player), SWFDEC_KEY_ESCAPE, 0);
+  }
+  swfmoz_player_invalidate (player);
+}
+
 static void
 swfmoz_player_notify (SwfmozPlayer *player, GParamSpec *pspec, gpointer unused)
 {
@@ -296,6 +375,25 @@ swfmoz_player_notify (SwfmozPlayer *player, GParamSpec *pspec, gpointer unused)
     swfmoz_player_invalidate (player);
   } else if (g_str_equal (pspec->name, "mouse-cursor")) {
     swfmoz_player_update_cursor (player);
+  } else if (g_str_equal (pspec->name, "fullscreen")) {
+    gboolean fullscreen = swfdec_player_get_fullscreen (SWFDEC_PLAYER (player));
+    if (fullscreen && player->fullscreen == NULL) {
+      GtkWidget *window, *child;
+      
+      player->fullscreen = window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+      child = swfdec_gtk_widget_new_fullscreen (SWFDEC_PLAYER (player));
+      gtk_container_add (GTK_CONTAINER (window), child);
+      gtk_widget_show_all (window);
+      gtk_window_fullscreen (GTK_WINDOW (window));
+      g_signal_connect (window, "destroy", 
+	  G_CALLBACK (swfmoz_player_fullscreen_destroyed), player);
+      gtk_widget_grab_focus (child);
+    } else if (!fullscreen && player->fullscreen != NULL) {
+      gtk_widget_destroy (player->fullscreen);
+      g_assert (player->fullscreen == NULL);
+    }
+  } else if (g_str_equal (pspec->name, "background-color")) {
+    swfmoz_player_update_background (player);
   }
 }
 
@@ -330,6 +428,12 @@ swfmoz_player_dispose (GObject *object)
 {
   SwfmozPlayer *player = SWFMOZ_PLAYER (object);
 
+  /* do this first or we'll get unhappy */
+  if (player->fullscreen != NULL) {
+    gtk_widget_destroy (player->fullscreen);
+    g_assert (player->fullscreen == NULL);
+  }
+
   if (player->menu) {
     g_signal_handlers_disconnect_matched (player, G_SIGNAL_MATCH_FUNC, 0, 0, NULL, 
 	swfmoz_player_menu_notify_playing, NULL);
@@ -362,6 +466,9 @@ swfmoz_player_dispose (GObject *object)
     player->loaders = NULL;
   }
 
+  g_object_unref (player->config);
+  player->config = NULL;
+
   G_OBJECT_CLASS (swfmoz_player_parent_class)->dispose (object);
 }
 
@@ -384,12 +491,12 @@ swfmoz_player_init (SwfmozPlayer *player)
   player->context = g_main_context_default ();
 
   player->loaders = GTK_TREE_MODEL (gtk_list_store_new (SWFMOZ_LOADER_N_COLUMNS,
-      SWFMOZ_TYPE_LOADER, G_TYPE_STRING, G_TYPE_STRING, 
-      G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, G_TYPE_UINT));
+      SWFMOZ_TYPE_LOADER, G_TYPE_STRING, G_TYPE_STRING,
+      G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, G_TYPE_STRING));
 }
 
 SwfdecPlayer *
-swfmoz_player_new (NPP instance, gboolean windowless)
+swfmoz_player_new (NPP instance, gboolean windowless, gboolean opaque)
 {
   SwfmozPlayer *ret;
   SwfdecSystem *system;
@@ -402,6 +509,8 @@ swfmoz_player_new (NPP instance, gboolean windowless)
       NULL);
   ret->instance = instance;
   ret->windowless = windowless;
+  ret->opaque = opaque;
+  ret->config = swfmoz_config_new ();
 
   return SWFDEC_PLAYER (ret);
 }
@@ -409,22 +518,36 @@ swfmoz_player_new (NPP instance, gboolean windowless)
 static void
 swfmoz_player_loaders_update (GtkListStore *store, GtkTreeIter *iter, SwfdecLoader *loader)
 {
-  glong percent;
+  goffset loaded, size;
   gboolean eof, error;
   const SwfdecURL *url;
   const char *url_string;
+  char *str_loaded, *str_size;
+  gchar *status;
+
+  loaded = swfdec_loader_get_loaded (loader);
+  size = swfdec_loader_get_size (loader);
 
-  percent = swfdec_loader_get_size (loader);
-  if (percent == 0) {
-    percent = 100;
-  } else if (percent < 0) {
-    percent = 50;
-  } else {
-    percent = 100 * swfdec_loader_get_loaded (loader) / percent;
-    percent = CLAMP (percent, 0, 100);
-  }
   /* FIXME: swfdec needs a function for this */
   g_object_get (G_OBJECT (loader), "eof", &eof, "error", &error, NULL);
+
+  if (error == TRUE) {
+    status = g_strdup("error");
+  } else {
+    str_loaded = g_format_size_for_display(loaded);
+    str_size = g_format_size_for_display(size);
+
+    if (size == loaded)
+      status = g_strdup_printf("%s", str_loaded);
+    else if (size < 0)
+      status = g_strdup_printf("at %s", str_loaded);
+    else
+      status = g_strdup_printf("%s of %s", str_loaded, str_size);
+
+    g_free (str_loaded);
+    g_free (str_size);
+  }
+
   url = swfdec_loader_get_url (loader);
   if (url) {
     url_string = swfdec_url_get_url (url);
@@ -441,8 +564,10 @@ swfmoz_player_loaders_update (GtkListStore *store, GtkTreeIter *iter, SwfdecLoad
     SWFMOZ_LOADER_COLUMN_EOF, eof,
     SWFMOZ_LOADER_COLUMN_ERROR, error,
     SWFMOZ_LOADER_COLUMN_TYPE, swfmoz_loader_get_data_type_string (loader),
-    SWFMOZ_LOADER_COLUMN_PERCENT_LOADED, (guint) percent,
+    SWFMOZ_LOADER_COLUMN_STATUS, status,
     -1);
+
+  g_free(status);
 }
 
 static gboolean
@@ -491,7 +616,12 @@ swfmoz_player_set_initial_stream (SwfmozPlayer *player, NPStream *stream)
     return FALSE;
   }
   swfdec_player_set_url (SWFDEC_PLAYER (player), url);
+  if (swfmoz_config_should_autoplay (player->config, url)) {
+    swfdec_gtk_player_set_playing (SWFDEC_GTK_PLAYER (player), TRUE);
+  }
   swfdec_url_free (url);
+  swfmoz_player_invalidate (player);
+
   return TRUE;
 }
 
@@ -518,24 +648,37 @@ swfmoz_player_set_target (SwfmozPlayer *player, GdkWindow *target,
   g_return_if_fail (SWFMOZ_IS_PLAYER (player));
   g_return_if_fail (target == NULL || GDK_IS_WINDOW (target));
 
-  if (player->target) {
-    g_object_unref (player->target);
+  if (target != player->target) {
+    if (player->target) {
+      g_object_unref (player->target);
+    }
+    player->target = target;
+    if (target) {
+      cairo_t *cr;
+      SwfdecRenderer *renderer;
+
+      g_object_ref (target);
+      cr = gdk_cairo_create (target);
+      renderer = swfdec_renderer_new_for_player (cairo_get_target (cr), 
+	  SWFDEC_PLAYER (player));
+      swfdec_player_set_renderer (SWFDEC_PLAYER (player), renderer);
+      g_object_unref (renderer);
+      cairo_destroy (cr);
+
+      swfdec_gtk_player_set_missing_plugins_window (SWFDEC_GTK_PLAYER (player), 
+	  gdk_window_get_toplevel (target));
+      swfmoz_player_update_cursor (player);
+      swfmoz_player_update_background (player);
+    } else {
+      swfdec_gtk_player_set_missing_plugins_window (SWFDEC_GTK_PLAYER (player), 
+	  NULL);
+    }
   }
-  player->target = target;
   player->target_rect.x = x;
   player->target_rect.y = y;
   player->target_rect.width = width;
   player->target_rect.height = height;
-  swfdec_player_set_size (SWFDEC_PLAYER (player), width - x, height - y);
-  if (target) {
-    g_object_ref (target);
-    swfdec_gtk_player_set_missing_plugins_window (SWFDEC_GTK_PLAYER (player), 
-	gdk_window_get_toplevel (target));
-    swfmoz_player_update_cursor (player);
-  } else {
-    swfdec_gtk_player_set_missing_plugins_window (SWFDEC_GTK_PLAYER (player), 
-	NULL);
-  }
+  swfdec_player_set_size (SWFDEC_PLAYER (player), width, height);
 }
 
 static void
@@ -606,20 +749,18 @@ swfdec_gtk_player_draw_pause (cairo_t *cr)
 }
 
 void
-swfmoz_player_render (SwfmozPlayer *player, GdkRegion *region)
+swfmoz_player_render (SwfmozPlayer *player, cairo_t *cr, GdkRegion *region)
 {
   GdkRectangle rect;
-  cairo_t *cr;
+  gboolean has_cr = cr != NULL;
 
   g_return_if_fail (SWFMOZ_IS_PLAYER (player));
   g_return_if_fail (!gdk_region_empty (region));
 
   /* first, remove the repainted stuff from the stuff that needs a redraw */
   if (player->repaint) {
-
     g_assert (player->repaint_source);
     gdk_region_subtract (player->repaint, region);
-
     if (gdk_region_empty (player->repaint)) {
       g_source_destroy (player->repaint_source);
       g_source_unref (player->repaint_source);
@@ -630,20 +771,33 @@ swfmoz_player_render (SwfmozPlayer *player, GdkRegion *region)
   }
 
   /* second, check if we have anything to draw */
-  if (player->target == NULL)
+  if (player->target == NULL || player->fullscreen)
     return;
 
-  /* paint it */
-  gdk_window_begin_paint_region (player->target, region);
-  cr = gdk_cairo_create (player->target);
+  if (!has_cr) {
+    gdk_window_begin_paint_region (player->target, region);
+    cr = gdk_cairo_create (player->target);
+  } else {
+    cairo_save (cr);
+  }
   gdk_cairo_region (cr, region);
   cairo_clip (cr);
+  /* paint it */
+  if (player->opaque) {
+    guint bgcolor = swfdec_player_get_background_color (SWFDEC_PLAYER (player));
+    cairo_set_source_rgb (cr, 
+	((bgcolor >> 16) & 0xFF) / 255.0,
+	((bgcolor >> 8) & 0xFF) / 255.0,
+	(bgcolor & 0xFF) / 255.0);
+    cairo_paint (cr);
+  }
   cairo_translate (cr, player->target_rect.x, player->target_rect.y);
   gdk_region_get_clipbox (region, &rect);
   swfdec_player_render (SWFDEC_PLAYER (player), cr, rect.x - player->target_rect.x,
       rect.y - player->target_rect.y, rect.width, rect.height);
   /* paint optional pause sign */
-  if (!swfdec_gtk_player_get_playing (SWFDEC_GTK_PLAYER (player))) {
+  if (!swfdec_gtk_player_get_playing (SWFDEC_GTK_PLAYER (player)) &&
+      swfdec_player_get_url (SWFDEC_PLAYER (player)) != NULL) {
     int w = player->target_rect.width;
     int h = player->target_rect.height;
     int len = MIN (w, h) * 4 / 5;
@@ -654,8 +808,24 @@ swfmoz_player_render (SwfmozPlayer *player, GdkRegion *region)
     cairo_scale (cr, len / 32.0, len / 32.0);
     swfdec_gtk_player_draw_pause (cr);
   }
-  cairo_destroy (cr);
-  gdk_window_end_paint (player->target);
+  if (!has_cr) {
+    cairo_destroy (cr);
+    gdk_window_end_paint (player->target);
+  } else {
+    cairo_restore (cr);
+  }
+}
+
+void
+swfmoz_player_set_allow_popups (SwfmozPlayer *player, gboolean allow)
+{
+  g_return_if_fail (SWFMOZ_IS_PLAYER (player));
+
+  swfdec_player_set_allow_fullscreen (SWFDEC_PLAYER (player), allow);
+  if (allow)
+    plugin_push_allow_popups (player->instance, TRUE);
+  else
+    plugin_pop_allow_popups (player->instance);
 }
 
 gboolean
@@ -675,9 +845,9 @@ swfmoz_player_mouse_press (SwfmozPlayer *player, int x, int y, guint button)
     return FALSE;
 
   if (swfdec_gtk_player_get_playing (SWFDEC_GTK_PLAYER (player))) {
-    plugin_push_allow_popups (player->instance, TRUE);
+    swfmoz_player_set_allow_popups (player, TRUE);
     ret = swfdec_player_mouse_press (SWFDEC_PLAYER (player), x, y, button);
-    plugin_pop_allow_popups (player->instance);
+    swfmoz_player_set_allow_popups (player, FALSE);
   }
   return ret;
 }
@@ -698,12 +868,13 @@ swfmoz_player_mouse_release (SwfmozPlayer *player, int x, int y, guint button)
     return FALSE;
 
   if (swfdec_gtk_player_get_playing (SWFDEC_GTK_PLAYER (player))) {
-    plugin_push_allow_popups (player->instance, TRUE);
+    swfmoz_player_set_allow_popups (player, TRUE);
     ret = swfdec_player_mouse_release (SWFDEC_PLAYER (player), x, y, button);
-    plugin_pop_allow_popups (player->instance);
+    swfmoz_player_set_allow_popups (player, FALSE);
   } else {
     if (button == 1) {
       swfdec_gtk_player_set_playing (SWFDEC_GTK_PLAYER (player), TRUE);
+      swfmoz_config_set_autoplay (player->config, swfdec_player_get_url (SWFDEC_PLAYER (player)), TRUE);
       ret = TRUE;
     } else {
       ret = FALSE;
